#include "Colors.h"
#include "Static.h"

Colors::Colors()
  : BufferedInputSource("Colors"),
    m_colorTimer(0),
    m_secondsPerColor(9)
{
  state = Task::Running;
}

void
Colors::loop()
{
  EVERY_N_SECONDS(1) {
    if (m_colorTimer > 0) {
      Log.verbose("colors: tick");
      m_colorTimer -= 1;

      if (m_colorTimer == 0) {
        rotate();
      }
    }
  }

  ConfigTaskMixin::loop();
}

void
Colors::handleEvent(const InputEvent& evt)
{
  if (evt.intent == InputEvent::SetScene) {
    setScene(evt.asString());
  }
  ConfigTaskMixin::handleEvent(evt);
}

void
Colors::setSequence(const char* seqName)
{
  bool found = false;
  int idx = 0;
  for(Sequence seq : m_sequences) {
    if (strcmp(seq.name.c_str(), seqName) == 0) {
      found = true;
      break;
    }
    idx++;
  }
  if (found) {
    Log.info("colors: Setting sequence to %s", seqName);
    m_idx = idx;
    rotate();
  } else {
    Log.warning("colors: Unknown sequence %s", seqName);
  }
}

void
Colors::rotate()
{
  if (!m_sequences.empty()) {
    Log.info("colors: rotate!") ;
    m_colorIdx += 1;
    m_colorIdx %= m_sequences[m_idx].colors.size();
    m_colorTimer = m_secondsPerColor;
    MainLoop::instance()->dispatch(InputEvent{InputEvent::SetColor, m_sequences[m_idx].colors[m_colorIdx]});
    //setEvent(InputEvent{InputEvent::SetColor, m_sequences[m_idx].colors[m_colorIdx]});
  } else {
    Log.info("colors: No colors to rotate");
  }
}

void
Colors::setScene(const char* sceneName)
{
  auto found = m_sceneMap.find(sceneName);
  if (found != m_sceneMap.end()) {
    Log.info("colors: Running scene %s", sceneName);
    setSequence(found->second);
  } else {
    Log.warning("colors: Unknown scene %s", sceneName);
    m_colorTimer = 0;
  }
}

void
Colors::handleConfigChange(const Configuration& cfg)
{
  Log.info("colors: reloading config");
  m_sequences.clear();
  m_colorIdx = 0;
  m_idx = 0;
  m_secondsPerColor = cfg.get("colors.secondsPerColor", 9);
  m_colorTimer = 0;
  JsonObject seqJson = cfg.get("colors.sequences");
  for(const auto seq : seqJson) {
    Log.info("colors: Found color sequence %s", seq.key().c_str());
    std::vector<CRGB> colors;
    for(JsonArray rgb : seq.value().as<JsonArray>()) {
      colors.emplace_back(CRGB(rgb[0], rgb[1], rgb[2]));
    }
    m_sequences.emplace_back(Sequence{
        seq.key().c_str(),
        std::move(colors)
    });
  }

  m_sceneMap.clear();
  JsonObject sceneJson = cfg.get("colors.scenes");
  for(const auto scene : sceneJson) {
    Log.info("colors: Found scene %s", scene.key().c_str());
    m_sceneMap[scene.key().c_str()] = scene.value().as<JsonString>().c_str();
  }

  rotate();
}

void
Colors::doSequences(Args& args, Print& out)
{
  out.println("Color sequences:");
  for(const Sequence& seq : m_sequences) {
    out.print("\t");
    out.println(seq.name.c_str());
  }
}

void
Colors::doSequence(Args& args, Print& out)
{
  setSequence(args[1].c_str());
}

void
Colors::doRotate(Args& args, Print& out)
{
  rotate();
}

const std::vector<Command>&
Colors::commands() const
{
  static const std::vector<Command> _commands = {
    {"sequences", &Colors::doSequences},
    {"sequence", &Colors::doSequence},
    {"rotate", &Colors::doRotate}
  };
  return _commands;
}

STATIC_TASK(Colors);
STATIC_ALLOC(Colors);
