#include "./JsonCoordinateMapping.h"

unsigned int
JsonCoordinateMapping::physicalPixelCount() const {
  int total = 0;
  for(int i = 0; i < strideCount; i++) {
    total += displayMap[i].length;
  }
  return total;
}

int
JsonCoordinateMapping::physicalCoordsToIndex(const PhysicalCoordinates localCoords) const {
  return displayMap[localCoords.x].physicalIdx + localCoords.y;
}

VirtualCoordinates
JsonCoordinateMapping::rotate(const VirtualCoordinates coords, uint8_t rot) const
{
  if (rot == 0) {
    return coords;
  } else if (rot == 1) {
    return VirtualCoordinates{coords.y, coords.x};
  } else if (rot == 2) {
    return VirtualCoordinates{255 - coords.y, 255 - coords.x};
  } else if (rot == 3) {
    return VirtualCoordinates{255 - coords.x, 255 - coords.y};
  } else {
    return coords;
  }
}

PhysicalCoordinates
JsonCoordinateMapping::virtualToPhysicalCoords(const VirtualCoordinates virtualCoords) const {
  // 0, 0 -> first pixel, first strand
  // 0, 255 -> first pixel, last strand
  // 255, 255 -> last pixel, last strand
  const VirtualCoordinates rotated = rotate(virtualCoords, rotation);
  const uint8_t spanIdx = scale8(strideCount-1, rotated.x);
  const uint8_t spanOffset = scale8(displayMap[spanIdx].length, rotated.y);
  return PhysicalCoordinates{spanIdx, spanOffset};
}

VirtualCoordinates 
JsonCoordinateMapping::physicalToVirtualCoords(const PhysicalCoordinates localCoords) const {
  int offset = localCoords.x;
  for(int i = 0; i < strideCount; i++) {
    if (offset > displayMap[i].length) {
      offset -= displayMap[i].length;
    } else {
      return VirtualCoordinates{i, offset};
    }
  }
  return VirtualCoordinates{0, 0};
}

void
JsonCoordinateMapping::load(const JsonArray& strideList)
{
  strideCount = strideList.size();
  uint16_t idx = 0;
  for(int i = 0; i < strideList.size();i++) {
    JsonObject strideObj = strideList[i];
    int length = strideObj["pixels"];
    int x = strideObj["x"];
    int y = strideObj["y"];
    bool reverse = strideObj["reverse"];
    displayMap[i] = Span{length, x, y, reverse, idx};
    idx += length;
    Log.info("config: Found stride (%d, %d): %d", x, y, length);
  }
}

void
JsonCoordinateMapping::loadDefault()
{
    displayMap[0] = Span{255, 0, 0, false, 0};
    strideCount = 1;
}
