#include "Cloudy.h"
#include "../Static.h"

Cloudy::Cloudy()
  : Figment("Cloudy")
{
}

void
Cloudy::loop()
{
  m_noiseOffset += 1;
  EVERY_N_MILLISECONDS(60) {
    m_hue.update(1);
  }
}

void
Cloudy::render(Display* dpy) const
{
  Surface sfc = Surface(dpy, {0, 0}, {255, 128});
  uint8_t noiseY = sin8(m_noiseOffset % 255);
  uint8_t noiseX = cos8(m_noiseOffset % 255);
  sfc.paintShader([=](CRGB& pixel, const VirtualCoordinates& coords, const PhysicalCoordinates, const VirtualCoordinates& surfaceCoords) {
      uint8_t brightness = inoise8(noiseX + surfaceCoords.x, noiseY + surfaceCoords.y);
      uint8_t saturation = inoise8(noiseY + surfaceCoords.y, noiseX + surfaceCoords.x);
      nblend(pixel, CHSV(m_hue, scale8(saturation, 128), brightness), 255 - surfaceCoords.y);
  });
  Surface horizon = Surface(dpy, {0, 64}, {255, 128});
  horizon.paintShader([=](CRGB& pixel, const VirtualCoordinates& coords, const PhysicalCoordinates, const VirtualCoordinates& surfaceCoords) {
      uint8_t brightness = inoise8(noiseX + surfaceCoords.x, noiseY + surfaceCoords.y);
      uint8_t saturation = inoise8(noiseY + surfaceCoords.y, noiseX + surfaceCoords.x);
      nblend(pixel, CHSV(m_hue + (saturation % 45), std::max((uint8_t)64, (uint8_t)saturation), brightness), surfaceCoords.y);
  });
}

void
Cloudy::handleEvent(const InputEvent& evt) {
  if (evt.intent == InputEvent::SetColor) {
    CHSV next = rgb2hsv_approximate(evt.asRGB());
    m_hue.set(next.h);
  }
}

STATIC_ALLOC(Cloudy);
STATIC_TASK(Cloudy);
