#include "Rain.h"
#include "../Static.h"

RainAnimation::RainAnimation() : Figment("Rain")
{
}

void
RainAnimation::render(Display* dpy) const
{
  Surface ground = Surface(dpy, {0, 64}, {255, 255});
  uint8_t noiseY = sin8(m_noiseOffset % 255);
  uint8_t noiseX = cos8(m_noiseOffset % 255);
  ground.paintShader([=](CRGB& pixel, const VirtualCoordinates& coords, const PhysicalCoordinates, const VirtualCoordinates& surfaceCoords) {
      uint8_t brightness = inoise8(noiseX + surfaceCoords.x, noiseY + surfaceCoords.y);
      uint8_t saturation = inoise8(noiseY + surfaceCoords.y, noiseX + surfaceCoords.x);
      nblend(pixel, CHSV(m_hue, saturation, brightness), 255 - surfaceCoords.y);
  });
  m_drops.render(dpy);
}

void
RainAnimation::loop()
{
  EVERY_N_MILLISECONDS(120) {
    m_drops.update();
  }
  EVERY_N_MILLISECONDS(60) {
    m_hue.update(1);
    m_curColor.h = m_hue;
  }
  m_noiseOffset += 1;
}

void
RainAnimation::handleEvent(const InputEvent& evt) {
  if (evt.intent == InputEvent::SetColor) {
    CHSV next = rgb2hsv_approximate(evt.asRGB());
    m_hue.set(next.h);
    m_drops.forEach([=](Raindrop& drop) {
      drop.nextColor = next;
    });
  }
}

STATIC_ALLOC(RainAnimation);
STATIC_TASK(RainAnimation);
