#include "./TestAnimation.h"
#include "../Static.h"
#include <FastLED.h>

void
TestAnimation::loop()
{
  m_x += 1;
  m_y += 1;
  m_hue += 1;
}

void
TestAnimation::handleEvent(const InputEvent& evt)
{
  if (evt.intent == InputEvent::SetColor) {
    m_color = evt.asRGB();
  }
}

void
TestAnimation::render(Display* dpy) const
{
  CRGB drawColor = blend(m_color, CHSV(m_hue, 255, 255), 128);
  if (m_mode == Raw) {
    for(unsigned int i = 0; i < dpy->pixelCount(); i++) {
      dpy->pixelAt(i) = drawColor;
    }
  } else if (m_mode == Top) {
    Surface{dpy, {0, 0}, {255, 0}} = drawColor;
  } else if (m_mode == Left) {
    Surface{dpy, {0, 0}, {0, 255}} = drawColor;
  } else if (m_mode == Right) {
    Surface{dpy, {255, 0}, {255, 255}} = drawColor;
  } else if (m_mode == Bottom) {
    Surface{dpy, {0, 255}, {255, 255}} = drawColor;
  } else if (m_mode == Outline) {
    // Draw red line on top row
    Surface{dpy, {0, 0}, {255, 0}} = CRGB{255, 0, 0};
    // Green line on first column
    Surface{dpy, {0, 0}, {0, 255}} = CRGB{0, 255, 0};
  } else if (m_mode == White) {
    Surface{dpy, {0, 0}, {255, 255}} = CRGB{255, 255, 255};
  } else if (m_mode == Red) {
    Surface{dpy, {0, 0}, {255, 255}} = CRGB{255, 0, 0};
  } else if (m_mode == Green) {
    Surface{dpy, {0, 0}, {255, 255}} = CRGB{0, 255, 0};
  } else if (m_mode == Blue) {
    Surface{dpy, {0, 0}, {255, 255}} = CRGB{0, 0, 255};
  } else if (m_mode == HSV) {
    Surface{dpy, {0, 0}, {255, 255}}.paintShader([=](CRGB& pixel, const VirtualCoordinates& coords, const PhysicalCoordinates&, const VirtualCoordinates& surfaceCoords) {
        pixel = CHSV(coords.x, coords.y, 255);
    });
  } else if (m_mode == RGB) {
    Surface{dpy, {0, 0}, {255, 255}}.paintShader([=](CRGB& pixel, const VirtualCoordinates& coords, const PhysicalCoordinates&, const VirtualCoordinates& surfaceCoords) {
        pixel = CRGB(coords.x, coords.y, (coords.x/2) + (coords.y/2));
    });
  }
  return;
}

void
TestAnimation::doMode(Args& args, Print& out)
{
  String s = args[1];
  if (s == "") {
    out.println("Available modes: raw outline white red green blue hsv rgb none top left right bottom");
    out.print("Current mode: ");
    out.println((int)m_mode);
  } else if (s == "raw") {
    m_mode = Raw;
  } else if (s == "outline") {
    m_mode = Outline;
  } else if (s == "white") {
    m_mode = White;
  } else if (s == "red") {
    m_mode = Red;
  } else if (s == "green") {
    m_mode = Green;
  } else if (s == "blue") {
    m_mode = Blue;
  } else if (s == "hsv") {
    m_mode = HSV;
  } else if (s == "rgb") {
    m_mode = RGB;
  } else if (s == "top") {
    m_mode = Top;
  } else if (s == "left") {
    m_mode = Left;
  } else if (s == "right") {
    m_mode = Right;
  } else if (s == "bottom") {
    m_mode = Bottom;
  } else if (s == "none") {
    m_mode = None;
  } else {
    out.println("Unknown test mode");
  }
}

const std::vector<Command>&
TestAnimation::commands() const
{
    static const std::vector<Command> _commands{
      {"test", &TestAnimation::doMode}
    };
    return _commands;
}

STATIC_ALLOC(TestAnimation);
STATIC_TASK(TestAnimation);
